import logging
from pathlib import Path
from pprint import pformat
import sqlite3

class DB:
    """! Docset database interface"""

    ## @var path
    #  path of the sqlite database

    def __init__(self, db_path: Path) -> None:
        """! Set member variables and create table

        @param db_path    path to the database file
        """
        
        self.path: Path = db_path
        self.create_table()
        
    def create_table(self) -> None:
        """! Create sqlite3 table at specified database file path

        The table will have the necessary constraints for a dash docset
        """

        con: sqlite3.Connection = sqlite3.connect(self.path)
        cur: sqlite3.Cursor = con.cursor()
        _ = cur.execute(
        """
        DROP TABLE IF EXISTS searchIndex
        """
        )
        _ = cur.execute(
        """
        CREATE TABLE IF NOT EXISTS searchIndex(id INTEGER PRIMARY KEY, name TEXT, 
        type TEXT, path TEXT);
        """
        )
        _ = cur.execute(
        """
        CREATE UNIQUE INDEX IF NOT EXISTS anchor ON searchIndex (name, type, path);
        """
        )
        con.close()

    def insert(self, name: str, type: str, page_path: str) -> None:
        """! Insert an index entry into the searchIndex table

        @param name         index entry name
        @param type         index entry type
        @param page_path    index entry page path
        """

        logging.debug("Inserting into " + str(self.path) + " with the following:")
        logging.debug("\tname = " + name)
        logging.debug("\ttype = " + type)
        logging.debug("\tpage_path = " + page_path)

        con = sqlite3.connect(self.path)
        cur = con.cursor()
        query = f"""
        INSERT INTO searchIndex(name, type, path) VALUES (\"{name}\",\"{type}\",
        \"{page_path}\");
        """

        try:
            _ = cur.execute(query)
            con.commit()
        except sqlite3.IntegrityError as e:
            logging.warning("Skipping query: " + pformat(e))
        con.close()



import argparse
from pathlib import Path

def get_argparse_template() -> argparse.ArgumentParser:
    """! generate base ArgumentParser object
    
    Takes a MANUAL_SOURCE argument and optional builddir.
    Add more arguments on top of the return object as you see fit

    """

    parser: argparse.ArgumentParser = argparse.ArgumentParser()
    _ = parser.add_argument("MANUAL_SOURCE", type=Path,
                            help="Path to the manual source")
    _ = parser.add_argument("-b", "--builddir", default=".", type=Path,
                            help="Directory to build the docset in")
    return parser



#!/usr/bin/env python3

import os
from pathlib import Path

class DocsetSkeleton:
    """! Easy docset structure hierarchy reference

    Generates a docset filestructure hierarhcy and provides relevant path
    references.
    """

    ## @var docset_dir
    #  path of the docset dir (`docset_name`.docset)

    ## @var contents_dir
    #  path of the Contents dir (under docset_dir)

    ## @var resources_dir
    #  path of the Resources dir (under contents_dir)
    
    ## @var documents_dir
    #  path of the Documents dir (under resources_dir).

    ## @var info_plist_file
    #  path of the Info.plist file (under contents_dir)

    ## @var index_file
    #  path of the docSet.dsidx file (under resources_dir)

    ## @var icon_file
    #  path of the icon.png file (under docset_dir)

    def __init__(
            self, 
            docset_name: str, 
            build_dir: Path | None = None
    ) -> None:
        """! Class initializer

        @param docset_name    docset_name, for use in constructing docset_dir
        @param build_dir      build directory, as determined by the 
                              ArgumentParser
        """
        self.docset_dir: Path
        self.contents_dir: Path
        self.resources_dir: Path
        self.documents_dir: Path
        self.info_plist_file: Path
        self.index_file: Path
        self.icon_file: Path

        if build_dir is None:
            build_dir = Path('.')

        # ensure valid build_dir
        if not build_dir.is_dir():
            print(f"Error: '{str(build_dir)}' is not a directory")
            exit()
        if not os.access(build_dir, os.W_OK):
            print(f"Error: '{str(build_dir)}' is not writable")
            exit()

        # create the directories
        self.docset_dir = build_dir.joinpath(f"{docset_name}.docset")
        self.contents_dir = self.docset_dir.joinpath("Contents")
        self.resources_dir = self.contents_dir.joinpath("Resources")
        self.documents_dir = self.resources_dir.joinpath("Documents")
        self.documents_dir.mkdir(parents = True, exist_ok = True)

        # create references to where files should go
        self.info_plist_file = self.contents_dir.joinpath("Info.plist")
        self.index_file = self.resources_dir.joinpath("docSet.dsidx")
        self.icon_file = self.docset_dir.joinpath("icon.png")



__all__ = [
        'DB',
        'DocsetSkeleton',
        'get_title',
        'get_argparse_template',
        'Gnu_Index_Terms',
        'Stylesheet_Setter'
]

from .argparse_template import get_argparse_template
from .db import DB
from .docset_skeleton import DocsetSkeleton
from .get_title import get_title
from .gnu import Gnu_Index_Terms
from .set_stylesheet import Stylesheet_Setter

# FIXME terrible practice, need to find a better way to hide these
del db
del docset_skeleton
del gnu
del set_stylesheet



#!/usr/bin/env python3

from bs4 import BeautifulSoup
import logging
import os
import requests
import sys

class Stylesheet_Setter:
    def __init__(self, css: bool, first_html_path: str) -> None:
        if css == True:
            # Download the css file and set each html file to use it
            web_css_path = self.get_css_path(first_html_path)
            local_css_path = None
            if not web_css_path is None:
                local_css_path = os.path.join(
                        os.path.dirname(first_html_path), 
                        "manual.css"
                )
                try:
                    r = requests.get(str(web_css_path))
                    _ = open(local_css_path, 'wb').write(r.content)
                except:
                    logging.warning(
                            f"Couldn't download css_path '{str(web_css_path)}'"
                    )
            if not local_css_path is None:
                for html_path in sys.argv[2:]:
                    self.stylesheet_replace(html_path)
            else:
                css = False

        if css == False:
            for html_path in sys.argv[2:]:
                self.stylesheet_remove(html_path)

    def get_css_path(self, html_path: str):
        soup = BeautifulSoup(open(html_path), 'html.parser')
        try:
            return soup.find('link', rel='stylesheet').get('href')
        except:
            return None

    def stylesheet_replace(self, html_path: str) -> None:
        with open(html_path) as f:
            soup = BeautifulSoup(f, 'html.parser')

            for stylesheet in soup.find_all('link', rel='stylesheet'):
                stylesheet.decompose()
            local_css_tag = soup.new_tag(
                    'link', 
                    rel='stylesheet', 
                    type='text/css', 
                    href='manual.css'
            )
            _ = soup.find_all('link')[-1].append(local_css_tag)

        with open(html_path, 'wb') as f:
            _ = f.write(soup.prettify("utf-8"))

    def stylesheet_remove(self, html_path: str) -> None:
        with open(html_path) as f:
            soup = BeautifulSoup(f, 'html.parser')

            for stylesheet in soup.find_all('link', rel='stylesheet'):
                stylesheet.decompose()

        with open(html_path, 'wb') as f:
            _ = f.write(soup.prettify("utf-8"))




from bs4 import BeautifulSoup
from pathlib import Path

def get_title(html_path: Path) -> str: 
    """! Get the title tag text from an html file
    
    @param html_path    path to html file to get title from

    Additionally, some sanitization is performed on the title: removal of 
    newline characters and replacement of double quotes with two double 
    quotes for sqlite insertion
    """

    soup = BeautifulSoup(open(html_path), 'html.parser')
    if soup.title is None:
        return ''
    title = soup.title.get_text()
    # Remove newline characters
    title = title.replace('\n', '')
    # Replace '"' with '""' for sake of support in sqlite insertion
    title = title.replace('"', '""')

    return title



#!/usr/bin/env python3

from bs4 import BeautifulSoup, Tag
import logging
from pathlib import Path
from pprint import pformat
import re

from .create_table import create_table
from .insert import insert

class Gnu_Index_Terms:
    def __init__(self, 
                 type: str, 
                 db_path: str, 
                 html_path: str, 
                 index_entry_class: str | None = None
    ) -> None:
        self.type: str = type
        self.db_path: str = db_path
        self.html_path: str = html_path
        self.index_entry_class: str | None = index_entry_class

        create_table(db_path)
        self.insert_index_terms()

    def insert_index_terms(self) -> None:
        soup: BeautifulSoup = BeautifulSoup(
                open(self.html_path), 'html.parser'
        )
        terms: list[Tag] | filter[Tag]
        if self.index_entry_class:
            terms = soup.find_all(class_=self.index_entry_class)
        else:
            terms = soup.find_all("td")

        for term in terms:
            logging.debug("Checking term " + pformat(term))
            logging.debug("\tget_text() produces " + pformat(term.get_text()))

        if self.index_entry_class:
            terms = filter(
                lambda x: re.search(
                    r'.*:$', 
                    x.get_text().lstrip().rstrip()
                ), 
                terms)

        for term in terms:
            self.insert_term(term)

    def insert_term(self, term: Tag) -> None:
        name: str
        if term.a:
            name = term.a.get_text()
            name = name.replace('"', '""')
            name = name.replace('\n', '')
            name = name.lstrip()
            name = re.sub(r'\s{3,}', ' ', name)

            page_path: Path = Path(self.html_path).parent.joinpath(
                    str(term.a['href'])
            ).resolve()
            # remove part of path leading up to actual interest
            for i in range(len(page_path.parts)):
                if page_path.parts[i:i+2] == (
                        "Contents", "Resources", "Documents"
                ):
                    page_path = Path(*page_path.parts[i+3:])
                    break

            insert(self.db_path, name, self.type, str(page_path))
